package tracing

import (
	"context"
	"github.com/go-kratos/kratos/v2/metadata"
	"strconv"
	"time"
)

const (
	ContextTraceId = "x-md-global-trace-id"
)

func SetId(traceId int, spanId int) int {
	snowflake := Snowflake{
		timestamp:    time.Now().UnixNano(),
		workerId:     1,
		datacenterId: int64(traceId),
		sequence:     int64(spanId),
	}
	return int(snowflake.NextVal())
}

func SetTraceId(ctx context.Context) (context.Context, int) {
	id := SetId(1, 0)
	ctx = context.WithValue(ctx, ContextTraceId, id)
	return metadata.AppendToClientContext(ctx, ContextTraceId, strconv.Itoa(id)), id
}

func GetTraceId(ctx context.Context) (context.Context, int) {
	if ctx == nil {
		return SetTraceId(context.TODO())
	}
	var traceId int
	if md, ok := metadata.FromServerContext(ctx); ok {
		id := md.Get(ContextTraceId)
		traceId, _ = strconv.Atoi(id)
		return ctx, traceId
	}

	if ctx.Value(ContextTraceId) != nil {
		traceId = ctx.Value(ContextTraceId).(int)
		return ctx, traceId
	}

	return SetTraceId(context.TODO())
}
